#!/usr/bin/env python
import argparse
import sys

# To use, run this from the top level of the Git repository tree:
# scripts/generate_sample_functions.py
#     --sample_autogen_h src/util/sample_autogen.h
#     --channelmixer_autogen_cpp
#     src/engine/channelmixer_autogen.cpp

BASIC_INDENT = 4

COPY_WITH_GAIN_METHOD_PATTERN = "copy%(i)dWithGain"


def copy_with_gain_method_name(i):
    return COPY_WITH_GAIN_METHOD_PATTERN % {"i": i}


RAMPING_GAIN_METHOD_PATTERN = "copy%(i)dWithRampingGain"


def copy_with_ramping_gain_method_name(i):
    return RAMPING_GAIN_METHOD_PATTERN % {"i": i}


def method_call(method_name, args):
    return "%(method_name)s(%(args)s)" % {
        "method_name": method_name,
        "args": ", ".join(args),
    }


def hanging_indent(base, groups, hanging_suffix, terminator, depth=0):
    return map(
        lambda line: " " * BASIC_INDENT * depth + line,
        map(
            "".join,
            zip(
                [base] + [" " * len(base)] * (len(groups) - 1),
                groups,
                [hanging_suffix] * (len(groups) - 1) + [terminator],
            ),
        ),
    )


def write_channelmixer_autogen(output, num_channels):
    output.append('#include "engine/channelmixer.h"')
    output.append('#include "util/sample.h"')
    output.append('#include "util/timer.h"')
    output.append("////////////////////////////////////////////////////////")
    output.append("// THIS FILE IS AUTO-GENERATED. DO NOT EDIT DIRECTLY! //")
    output.append("// SEE tools/generate_sample_functions.py             //")
    output.append("////////////////////////////////////////////////////////")
    output.append("")
    output.append("// static")

    def write_applyeffectsandmixchannels(inplace, output):
        header = (
            "void ChannelMixer::applyEffects" "%(inplace)sAndMixChannels("
        ) % {"inplace": "InPlace" if inplace else ""}
        args = [
            "const EngineMaster::GainCalculator& gainCalculator",
            "QVarLengthArray<EngineMaster::ChannelInfo*, "
            "kPreallocatedChannels>* activeChannels",
            "QVarLengthArray<EngineMaster::GainCache, "
            "kPreallocatedChannels>* channelGainCache",
            "CSAMPLE* pOutput",
            "const ChannelHandle& outputHandle",
            "unsigned int iBufferSize",
            "unsigned int iSampleRate",
            "EngineEffectsManager* pEngineEffectsManager",
        ]
        output.extend(hanging_indent(header, args, ",", ") {"))

        def write(data, depth=0):
            output.append(" " * (BASIC_INDENT * depth) + data)

        if inplace:
            write("// Signal flow overview:", depth=1)
            write("// 1. Calculate gains for each channel", depth=1)
            write(
                "// 2. Pass each channel's calculated gain and input buffer "
                "to pEngineEffectsManager, which then:",
                depth=1,
            )
            write(
                "//    A) Applies the calculated gain to the channel buffer, "
                "modifying the original input buffer",
                depth=1,
            )
            write(
                "//    B) Applies effects to the buffer, modifying the "
                "original input buffer",
                depth=1,
            )
            write(
                "// 4. Mix the channel buffers together to make pOutput, "
                "overwriting the pOutput buffer from the last engine callback",
                depth=1,
            )
        else:
            write("// Signal flow overview:", depth=1)
            write("// 1. Clear pOutput buffer", depth=1)
            write("// 2. Calculate gains for each channel", depth=1)
            write(
                "// 3. Pass each channel's calculated gain and input buffer "
                "to pEngineEffectsManager, which then:",
                depth=1,
            )
            write(
                "//     A) Copies each channel input buffer to a temporary "
                "buffer",
                depth=1,
            )
            write("//     B) Applies gain to the temporary buffer", depth=1)
            write(
                "//     C) Processes effects on the temporary buffer", depth=1
            )
            write("//     D) Mixes the temporary buffer into pOutput", depth=1)
            write(
                "// The original channel input buffers are not modified.",
                depth=1,
            )

        write("int totalActive = activeChannels->size();", depth=1)
        if not inplace:
            write("SampleUtil::clear(pOutput, iBufferSize);", depth=1)
        write("if (totalActive == 0) {", depth=1)
        write(
            (
                '//ScopedTimer t("EngineMaster::applyEffects%(inplace)s'
                'AndMixChannels_0active");'
            )
            % {"inplace": "InPlace" if inplace else ""},
            depth=2,
        )
        if inplace:
            write("SampleUtil::clear(pOutput, iBufferSize);", depth=2)
        for i in range(1, num_channels + 1):
            write("} else if (totalActive == %d) {" % i, depth=1)
            write(
                (
                    '//ScopedTimer t("EngineMaster::applyEffects%(inplace)s'
                    'AndMixChannels_%(i)dactive");'
                )
                % {"inplace": "InPlace" if inplace else "", "i": i},
                depth=2,
            )
            write("CSAMPLE_GAIN oldGain[%(i)d];" % {"i": i}, depth=2)
            write("CSAMPLE_GAIN newGain[%(i)d];" % {"i": i}, depth=2)
            for j in range(i):
                write(
                    (
                        "EngineMaster::ChannelInfo* pChannel%(j)d = "
                        "activeChannels->at(%(j)d);"
                    )
                    % {"j": j},
                    depth=2,
                )
                write(
                    "const int channelIndex%(j)d = pChannel%(j)d->m_index;"
                    % {"j": j},
                    depth=2,
                )
                write(
                    (
                        "EngineMaster::GainCache& gainCache%(j)d = "
                        "(*channelGainCache)[channelIndex%(j)d];"
                    )
                    % {"j": j},
                    depth=2,
                )
                write(
                    "oldGain[%(j)d] = gainCache%(j)d.m_gain;" % {"j": j},
                    depth=2,
                )
                write("if (gainCache%(j)d.m_fadeout) {" % {"j": j}, depth=2)
                write("newGain[%(j)d] = 0;" % {"j": j}, depth=3)
                write("gainCache%(j)d.m_fadeout = false;" % {"j": j}, depth=3)
                write("} else {", depth=2)
                write(
                    "newGain[%(j)d] = gainCalculator.getGain(pChannel%(j)d);"
                    % {"j": j},
                    depth=3,
                )
                write("}", depth=2)
                write(
                    "gainCache%(j)d.m_gain = newGain[%(j)d];" % {"j": j},
                    depth=2,
                )
                write(
                    "CSAMPLE* pBuffer%(j)d = pChannel%(j)d->m_pBuffer;"
                    % {"j": j},
                    depth=2,
                )

            if inplace:
                write("// Process effects for each channel in place", depth=2)
            else:
                write(
                    "// Process effects for each channel and mix the "
                    "processed signal into pOutput",
                    depth=2,
                )
            for j in range(i):
                if inplace:
                    write(
                        (
                            "pEngineEffectsManager->processPostFaderInPlace("
                            "pChannel%(j)d->m_handle, outputHandle, "
                            "pBuffer%(j)d, iBufferSize, iSampleRate, "
                            "pChannel%(j)d->m_features, oldGain[%(j)d], "
                            "newGain[%(j)d]);"
                        )
                        % {"j": j},
                        depth=2,
                    )
                else:
                    write(
                        (
                            "pEngineEffectsManager->processPostFaderAndMix("
                            "pChannel%(j)d->m_handle, outputHandle, "
                            "pBuffer%(j)d, pOutput, iBufferSize, iSampleRate, "
                            "pChannel%(j)d->m_features, oldGain[%(j)d], "
                            "newGain[%(j)d]);"
                        )
                        % {"j": j},
                        depth=2,
                    )

            if inplace:
                write(
                    "// Mix the effected channel buffers together to replace "
                    "the old pOutput from the last engine callback",
                    depth=2,
                )
                write(
                    "for (unsigned int i = 0; i < iBufferSize; ++i) {", depth=2
                )
                line = "pOutput[i] = pBuffer0[i]"
                for k in range(1, i):
                    line += " + pBuffer%(k)d[i]" % {"k": k}
                line += ";"
                write(line, depth=3)
                write("}", depth=2)

        write("} else {", depth=1)
        write(
            (
                '//ScopedTimer t("EngineMaster::applyEffects%(inplace)s'
                'AndMixChannels_Over32active");'
            )
            % {"inplace": "InPlace" if inplace else ""},
            depth=2,
        )
        if inplace:
            write("SampleUtil::clear(pOutput, iBufferSize);", depth=2)
        write("for (int i = 0; i < activeChannels->size(); ++i) {", depth=2)
        write(
            "EngineMaster::ChannelInfo* pChannelInfo = activeChannels->at(i);",
            depth=3,
        )

        write("const int channelIndex = pChannelInfo->m_index;", depth=3)
        write(
            "EngineMaster::GainCache& gainCache = "
            "(*channelGainCache)[channelIndex];",
            depth=3,
        )
        write("CSAMPLE_GAIN oldGain = gainCache.m_gain;", depth=3)
        write("CSAMPLE_GAIN newGain;", depth=3)
        write("if (gainCache.m_fadeout) {", depth=3)
        write("newGain = 0;", depth=4)
        write("gainCache.m_fadeout = false;", depth=4)
        write("} else {", depth=3)
        write("newGain = gainCalculator.getGain(pChannelInfo);", depth=4)
        write("}", depth=3)
        write("gainCache.m_gain = newGain;", depth=3)

        write("CSAMPLE* pBuffer = pChannelInfo->m_pBuffer;", depth=3)
        if inplace:
            write(
                "pEngineEffectsManager->processPostFaderInPlace("
                "pChannelInfo->m_handle, outputHandle, pBuffer, iBufferSize, "
                "iSampleRate, pChannelInfo->m_features, oldGain, newGain);",
                depth=3,
            )
            write("SampleUtil::add(pOutput, pBuffer, iBufferSize);", depth=3)
        else:
            write(
                "pEngineEffectsManager->processPostFaderAndMix("
                "pChannelInfo->m_handle, outputHandle, pBuffer, pOutput, "
                "iBufferSize, iSampleRate, pChannelInfo->m_features, oldGain, "
                "newGain);",
                depth=3,
            )

        write("}", depth=2)
        write("}", depth=1)
        output.append("}")

    write_applyeffectsandmixchannels(False, output)
    write_applyeffectsandmixchannels(True, output)


def write_sample_autogen(output, num_channels):
    output.append("#ifndef MIXXX_UTIL_SAMPLEAUTOGEN_H")
    output.append("#define MIXXX_UTIL_SAMPLEAUTOGEN_H")
    output.append("////////////////////////////////////////////////////////")
    output.append("// THIS FILE IS AUTO-GENERATED. DO NOT EDIT DIRECTLY! //")
    output.append("// SEE scripts/generate_sample_functions.py           //")
    output.append("////////////////////////////////////////////////////////")

    for i in range(1, num_channels + 1):
        copy_with_gain(output, 0, i)
        copy_with_ramping_gain(output, 0, i)

    output.append("#endif /* MIXXX_UTIL_SAMPLEAUTOGEN_H */")


def copy_with_gain(output, base_indent_depth, num_channels):
    def write(data, depth=0):
        output.append(
            " " * (BASIC_INDENT * (depth + base_indent_depth)) + data
        )

    header = "static inline void %s(" % copy_with_gain_method_name(
        num_channels
    )
    arg_groups = (
        ["CSAMPLE* M_RESTRICT pDest"]
        + [
            "const CSAMPLE* M_RESTRICT pSrc%(i)d, CSAMPLE_GAIN gain%(i)d"
            % {"i": i}
            for i in range(num_channels)
        ]
        + ["int iNumSamples"]
    )

    output.extend(
        hanging_indent(header, arg_groups, ",", ") {", depth=base_indent_depth)
    )

    for i in range(num_channels):
        write("if (gain%(i)d == CSAMPLE_GAIN_ZERO) {" % {"i": i}, depth=1)
        if num_channels > 1:
            args = (
                ["pDest"]
                + [
                    "pSrc%(i)d, gain%(i)d" % {"i": j}
                    for j in range(num_channels)
                    if i != j
                ]
                + ["iNumSamples"]
            )
            write(
                "%s;"
                % method_call(
                    copy_with_gain_method_name(num_channels - 1), args
                ),
                depth=2,
            )
        else:
            write("clear(pDest, iNumSamples);", depth=2)
        write("return;", depth=2)
        write("}", depth=1)

    write("// note: LOOP VECTORIZED.", depth=1)
    write("for (int i = 0; i < iNumSamples; ++i) {", depth=1)
    terms = [
        "pSrc%(i)d[i] * gain%(i)d" % {"i": i} for i in range(num_channels)
    ]
    assign = "pDest[i] = "
    output.extend(hanging_indent(assign, terms, " +", ";", depth=2))

    write("}", depth=1)
    write("}")


def copy_with_ramping_gain(output, base_indent_depth, num_channels):
    def write(data, depth=0):
        output.append(
            " " * (BASIC_INDENT * (depth + base_indent_depth)) + data
        )

    header = "static inline void %s(" % copy_with_ramping_gain_method_name(
        num_channels
    )
    arg_groups = (
        ["CSAMPLE* M_RESTRICT pDest"]
        + [
            (
                "const CSAMPLE* M_RESTRICT pSrc%(i)d, "
                "CSAMPLE_GAIN gain%(i)din, CSAMPLE_GAIN gain%(i)dout"
            )
            % {"i": i}
            for i in range(num_channels)
        ]
        + ["int iNumSamples"]
    )

    output.extend(
        hanging_indent(header, arg_groups, ",", ") {", depth=base_indent_depth)
    )

    for i in range(num_channels):
        write(
            (
                "if (gain%(i)din == CSAMPLE_GAIN_ZERO && "
                "gain%(i)dout == CSAMPLE_GAIN_ZERO) {"
            )
            % {"i": i},
            depth=1,
        )
        if num_channels > 1:
            args = (
                ["pDest"]
                + [
                    "pSrc%(i)d, gain%(i)din, gain%(i)dout" % {"i": j}
                    for j in range(num_channels)
                    if i != j
                ]
                + ["iNumSamples"]
            )
            write(
                "%s;"
                % method_call(
                    copy_with_ramping_gain_method_name(num_channels - 1), args
                ),
                depth=2,
            )
        else:
            write("clear(pDest, iNumSamples);", depth=2)
        write("return;", depth=2)
        write("}", depth=1)

    for i in range(num_channels):
        write(
            (
                "const CSAMPLE_GAIN gain_delta%(i)d = "
                "(gain%(i)dout - gain%(i)din) / (iNumSamples / 2);"
            )
            % {"i": i},
            depth=1,
        )
        write(
            (
                "const CSAMPLE_GAIN start_gain%(i)d = "
                "gain%(i)din + gain_delta%(i)d;"
            )
            % {"i": i},
            depth=1,
        )

    write("// note: LOOP VECTORIZED.", depth=1)
    write("for (int i = 0; i < iNumSamples / 2; ++i) {", depth=1)

    for i in range(num_channels):
        write(
            (
                "const CSAMPLE_GAIN gain%(i)d = "
                "start_gain%(i)d + gain_delta%(i)d * i;"
            )
            % {"i": i},
            depth=2,
        )

    terms1 = []
    terms2 = []
    for i in range(num_channels):
        terms1.append("pSrc%(i)d[i * 2] * gain%(i)d" % {"i": i})
        terms2.append("pSrc%(i)d[i * 2 + 1] * gain%(i)d" % {"i": i})

    assign1 = "pDest[i * 2] = "
    assign2 = "pDest[i * 2 + 1] = "

    output.extend(hanging_indent(assign1, terms1, " +", ";", depth=2))
    output.extend(hanging_indent(assign2, terms2, " +", ";", depth=2))

    write("}", depth=1)
    write("}")


def main(args):
    sampleutil_output_lines = []
    write_sample_autogen(sampleutil_output_lines, args.max_channels)

    output = (
        open(args.sample_autogen_h, "w")
        if args.sample_autogen_h
        else sys.stdout
    )
    output.write("\n".join(sampleutil_output_lines) + "\n")

    channelmixer_output_lines = []
    write_channelmixer_autogen(channelmixer_output_lines, args.max_channels)

    output = (
        open(args.channelmixer_autogen_cpp, "w")
        if args.channelmixer_autogen_cpp
        else sys.stdout
    )
    output.write("\n".join(channelmixer_output_lines) + "\n")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Auto-generate sample processing and mixing functions.",
        epilog=(
            "Example Call:"
            "./generate_sample_functions.py --sample_autogen_h "
            "../src/util/sample_autogen.h --channelmixer_autogen_cpp "
            "../src/engine/channelmixer_autogen.cpp"
        ),
    )
    parser.add_argument("--sample_autogen_h")
    parser.add_argument("--channelmixer_autogen_cpp")
    parser.add_argument("--max_channels", type=int, default=32)
    args = parser.parse_args()
    main(args)



#!/usr/bin/python

#############################################################################
#                          make_xone.py  - Ugly script which automatically
#                          creates a midi mapping for the Xone K2.
#                          Lots of this is boilerplate...
#                             -------------------
#    begin                : Fri Aug 31 18:45:00 EDT 2012
#    copyright            : (C) 2012 Owen Williams
#    email                : owilliams@mixxx.org
#############################################################################


import sys
import getopt


def help():
    print("usage: make_xone [args] outputfilename")
    print("optional args: ")
    print("          --4decks     ", end="")
    print("to generate a layout for 4 deck-enabled versions of mixxx")
    print("          --mastersync ", end="")
    print("to generate alternative layout for master_sync testing")


if len(sys.argv) < 2:
    help()
    sys.exit(1)

MASTER_SYNC_LAYOUT = False
MAX_DECKS = 2
fname = ""
DECK_ORDER = range(0, 2)

try:
    opts, args = getopt.getopt(sys.argv[1:], "", ["mastersync", "4decks"])
    for o, a in opts:
        if o == "--mastersync":
            # This is an alternative layout for my work on master sync
            MASTER_SYNC_LAYOUT = True
        elif o == "--4decks":
            MAX_DECKS = 4
            DECK_ORDER = (2, 0, 1, 3)
    if len(args) > 0:
        fname = args[0]
except Exception as e:
    print(str(e))
    help()
    sys.exit(1)

if fname == "":
    help()
    sys.exit(1)

try:
    fh = open(fname, "w")
except Exception as e:
    print("Error opening file for write:", str(e))
    sys.exit(1)

button_definitions = """0x34	0x35	0x36	0x37
0x58	0x59	0x5A	0x5B
0x7C	0x7d	0x7e	0x7f


0x30	0x31	0x32	0x33
0x54	0x55	0x56	0x57
0x78	0x79	0x7A	0x7b



0x2c	0x2d	0x2e	0x2f
0x50	0x51	0x52	0x53
0x74	0x75	0x76	0x77


0x28	0x29	0x2a	0x2b
0x4c	0x4d	0x4e	0x4f
0x70	0x71	0x72	0x73



0x24	0x25	0x26	0x27
0x48	0x49	0x4a	0x4b
0x6c	0x6d	0x6e	0x6f


0x20	0x21	0x22	0x23
0x44	0x45	0x46	0x47
0x68	0x69	0x6a	0x6b


0x1c	0x1d	0x1e	0x1f
0x40	0x41	0x42	0x43
0x64	0x65	0x66	0x67


0x18	0x19	0x1A	0x1b
0x3c	0x3d	0x3e	0x3f
0x60	0x61	0x62	0x63"""

controls = [
    "spinknob",
    "knoblight1",
    "knoblight2",
    "knoblight3",
    "button1",
    "button2",
    "button3",
    "button4",
]
controls.reverse()
colors = ["red", "orange", "green"]
cur_color = 0

# {'spinknob': [{'red':1,'orange':2,'green':3},
#               {'red':4,'orange':5,'green':6}, ...

midi = {}
current_control = controls.pop()
for line in button_definitions.split("\n"):
    if len(line.strip()) == 0:
        continue

    hexlist = line.split()

    if current_control not in midi:
        midi[current_control] = [{}, {}, {}, {}]

    for i, h in enumerate(hexlist):
        midi[current_control][i][colors[cur_color]] = h

    cur_color = (cur_color + 1) % len(colors)
    if cur_color == 0:
        try:
            current_control = controls.pop()
        except Exception:
            break

# ok now we have a mapping between control and hex key

# now we need midi CC's -- these just increment simply

cc_controls = ["spinknob", "eq1", "eq2", "eq3", "slider"]
cc_code = 0
midi_cc = {}
for cc in cc_controls:
    midi_cc[cc] = []
    for i in range(0, 4):
        midi_cc[cc].append("0x%x" % cc_code)
        cc_code += 1


# now we need a mapping of control and controlobject

# controls = ['spinknob', 'knoblight1', 'knoblight2', 'knoblight3', 'button1',
#             'button2', 'button3', 'button4']


##################################################
# Actual stuff you can edit!                     #
##################################################

# Note that EQ control objects are munged in code to produce the correct
# control objects as of Mixxx 2.0.

cc_mapping = {
    "spinknob": ["XoneK2.encoderJog", "<Script-Binding/>"],
    "eq1": ["filterHigh", "<normal/>"],
    "eq2": ["filterMid", "<normal/>"],
    "eq3": ["filterLow", "<normal/>"],
    "slider": ["volume", "<normal/>"],
}

button_mapping = {
    "spinknob": ["beatsync", "<button/>"],
    "knoblight1": ["filterHighKill", "<normal/>"],
    "knoblight2": ["filterMidKill", "<normal/>"],
    "knoblight3": ["filterLowKill", "<normal/>"],
    "button1": {
        "red": ["pfl", "<normal/>"],
        "orange": ["beatloop_4", "<button/>"],
        "green": ["hotcue_1_activate", "<button/>"],
    },
    "button2": {
        "red": ["keylock", "<normal/>"],
        "orange": ["loop_double", "<button/>"],
        "green": ["hotcue_2_activate", "<button/>"],
    },
    "button3": {
        "red": ["cue_default", "<button/>"],
        "orange": ["loop_halve", "<button/>"],
        "green": ["hotcue_3_activate", "<button/>"],
    },
    "button4": {
        "red": ["play", "<normal/>"],
        "orange": ["reloop_exit", "<button/>"],
        "green": ["hotcue_4_activate", "<button/>"],
    },
}

if MASTER_SYNC_LAYOUT:
    button_mapping = {
        "spinknob": ["XoneK2.encoderButton", "<Script-Binding/>"],
        "knoblight1": ["keylock", "<button/>"],
        "knoblight2": ["quantize", "<normal/>"],
        "knoblight3": ["filterLowKill", "<normal/>"],
        "button1": {
            "red": ["pfl", "<button/>"],
            "orange": ["beatloop_4", "<button/>"],
            "green": ["hotcue_1_activate", "<button/>"],
        },
        "button2": {
            "red": ["sync_enabled", "<button/>"],
            "orange": ["loop_double", "<button/>"],
            "green": ["hotcue_2_activate", "<button/>"],
        },
        "button3": {
            "red": ["cue_default", "<button/>"],
            "orange": ["loop_halve", "<button/>"],
            "green": ["hotcue_3_activate", "<button/>"],
        },
        "button4": {
            "red": ["play", "<button/>"],
            "orange": ["reloop_exit", "<button/>"],
            "green": ["hotcue_4_activate", "<button/>"],
        },
    }


light_mapping = {  # 'spinknob':'jog',
    "knoblight1": "filterHighKill",
    "knoblight2": "filterMidKill",
    "knoblight3": "filterLowKill",
    "button1": {
        "red": "pfl",
        "orange": "beatloop_4",
        "green": "hotcue_1_enabled",
    },
    "button2": {
        "red": "keylock",
        "orange": "loop_double",
        "green": "hotcue_2_enabled",
    },
    "button3": {
        "red": "cue_default",
        "orange": "loop_halve",
        "green": "hotcue_3_enabled",
    },
    "button4": {
        "red": "play",
        "orange": "loop_enabled",
        "green": "hotcue_4_enabled",
    },
}

if MASTER_SYNC_LAYOUT:
    light_mapping = {  # 'spinknob':'jog',
        "knoblight1": "keylock",
        "knoblight2": "quantize",
        "knoblight3": "filterLowKill",
        "button1": {
            "red": "pfl",
            "orange": "beatloop_4",
            "green": "hotcue_1_enabled",
        },
        "button2": {
            "red": "sync_enabled",
            "orange": "loop_double",
            "green": "hotcue_2_enabled",
        },
        "button3": {
            "red": "cue_default",
            "orange": "loop_halve",
            "green": "hotcue_3_enabled",
        },
        "button4": {
            "red": "play",
            "orange": "loop_enabled",
            "green": "hotcue_4_enabled",
        },
    }


# these aren't worth automating
master_knobs = """            <control>
                <group>[Playlist]</group>
                <key>XoneK2.rightBottomKnob</key>
                <status>0xBF</status>
                <midino>0x15</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
            <control>
                <group>[Playlist]</group>
                <key>LoadSelectedIntoFirstStopped</key>
                <status>0x9F</status>
                <midino>0x0E</midino>
                <options>
                    <normal/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.leftBottomKnob</key>
                <status>0xBF</status>
                <midino>0x14</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.shift_on</key>
                <status>0x9F</status>
                <midino>0xF</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.shift_on</key>
                <status>0x8F</status>
                <midino>0xF</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>"""

if MASTER_SYNC_LAYOUT:
    master_knobs = """            <control>
                <group>[Playlist]</group>
                <key>XoneK2.rightBottomKnob</key>
                <status>0xBF</status>
                <midino>0x15</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
             <control>
                <group>[Playlist]</group>
                <key>LoadSelectedIntoFirstStopped</key>
                <status>0x9F</status>
                <midino>0x0E</midino>
                <options>
                    <normal/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.leftBottomKnob</key>
                <status>0xBF</status>
                <midino>0x14</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.shift_on</key>
                <status>0x9F</status>
                <midino>0xF</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>
            <control>
                <group>[Master]</group>
                <key>XoneK2.shift_on</key>
                <status>0x8F</status>
                <midino>0xF</midino>
                <options>
                    <Script-Binding/>
                </options>
            </control>"""


xml = []
xml.append(
    """<?xml version='1.0' encoding='utf-8'?>
<!-- This file automatically generated by make_xone.py. -->
<MixxxControllerPreset mixxxVersion="" schemaVersion="1">
    <info>
        <name>Allen&amp;Heath Xone K2</name>
        <author>Owen Williams</author>
        <description>For this mapping to work:
- Set Xone:K2 midi channel to 16;
- Set Xone:K2 Latching Layers state to "Switch Matrix" (state 2).
(See product manual for details.)</description>
    </info>
    <controller id="XONE:K2">
        <scriptfiles>
            <file filename="Xone-K2-scripts.js" functionprefix="XoneK2"/>
        </scriptfiles>
        <controls>"""
)


xml.append("<!-- CC Controls (knobs and sliders) -->")

# for i, deck in enumerate(DECK_ORDER):
#    xml.append("""            <control>
#                <group>[Channel%i]</group>
#                <key>XoneK2.encoderJog%i</key>
#                <status>0xBF</status>
#                <midino>%s</midino>
#                <options>
#                    <Script-Binding/>
#                </options>
#            </control>""" % (deck+1, deck+1, midi_cc['spinknob'][i]))


########################################
########################################
########################################
# ok back to boilerplate...


def get_group_name(channel, key):
    """Optionally munge group name if an EQ"""
    if "filter" in key:
        return "[EqualizerRack1_[Channel%d]_Effect1]" % channel
    return "[Channel%d]" % channel


def get_key_name(key):
    """Optionally munge key name if an EQ"""
    if key == "filterLow":
        return "parameter1"
    if key == "filterMid":
        return "parameter2"
    if key == "filterHigh":
        return "parameter3"
    if key == "filterLowKill":
        return "button_parameter1"
    if key == "filterMidKill":
        return "button_parameter2"
    if key == "filterLowKill":
        return "button_parameter3"
    return key


# cc controls (no latching needed)
for i, channel in enumerate(DECK_ORDER):
    for cc in cc_mapping:
        xml.append(
            """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0xBF</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
            % (
                get_group_name(channel + 1, cc_mapping[cc][0]),
                get_key_name(cc_mapping[cc][0]),
                midi_cc[cc][i],
                cc_mapping[cc][1],
            )
        )

# Spin Knob buttons (no latching needed)
for i, channel in enumerate(DECK_ORDER):
    xml.append(
        """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x9F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
        % (
            get_group_name(channel + 1, button_mapping["spinknob"][0]),
            get_key_name(button_mapping["spinknob"][0]),
            midi["spinknob"][i]["red"],
            button_mapping["spinknob"][1],
        )
    )
    xml.append(
        """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x8F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
        % (
            get_group_name(channel + 1, button_mapping["spinknob"][0]),
            get_key_name(button_mapping["spinknob"][0]),
            midi["spinknob"][i]["red"],
            button_mapping["spinknob"][1],
        )
    )

xml.append("<!-- Upper Buttons -->")
# knoblight buttons (no latching)
for j, channel in enumerate(DECK_ORDER):
    for knob in ["knoblight%i" % i for i in range(1, 4)]:
        xml.append(
            """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x9F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
            % (
                get_group_name(channel + 1, button_mapping[knob][0]),
                get_key_name(button_mapping[knob][0]),
                midi[knob][j]["red"],
                button_mapping[knob][1],
            )
        )
        xml.append(
            """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x8F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
            % (
                get_group_name(channel + 1, button_mapping[knob][0]),
                get_key_name(button_mapping[knob][0]),
                midi[knob][j]["red"],
                button_mapping[knob][1],
            )
        )

xml.append("<!-- Lower Button Grid -->")

# buttons
for j, channel in enumerate(DECK_ORDER):
    for latch in ["red", "orange", "green"]:
        for button in ["button%i" % i for i in range(1, 5)]:
            xml.append(
                """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x9F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
                % (
                    get_group_name(
                        channel + 1, button_mapping[button][latch][0]
                    ),
                    get_key_name(button_mapping[button][latch][0]),
                    midi[button][j][latch],
                    button_mapping[button][latch][1],
                )
            )
            xml.append(
                """            <control>
                <group>%s</group>
                <key>%s</key>
                <status>0x8F</status>
                <midino>%s</midino>
                <options>
                    %s
                </options>
            </control>"""
                % (
                    get_group_name(
                        channel + 1, button_mapping[button][latch][0]
                    ),
                    get_key_name(button_mapping[button][latch][0]),
                    midi[button][j][latch],
                    button_mapping[button][latch][1],
                )
            )

xml.append("<!-- Special Case Knobs / buttons -->")
# a couple custom entries:
xml.append(master_knobs)


# done with controls

xml.append(
    """        </controls>
        <outputs>"""
)

# ok now the lights
if "spinknob" in light_mapping:
    for i, channel in enumerate(DECK_ORDER):
        xml.append(
            """            <output>
                    <group>%s</group>
                    <key>%s</key>
                    <status>0x9F</status>
                    <midino>%s</midino>
                    <minimum>0.5</minimum>
                </output>"""
            % (
                get_group_name(channel + 1, light_mapping["spinknob"]),
                get_key_name(light_mapping["spinknob"]),
                midi["spinknob"][i]["red"],
            )
        )


xml.append("<!-- Knob lights -->")
# knoblight buttons (no latching)
for j, channel in enumerate(DECK_ORDER):
    for knob in ["knoblight%i" % i for i in range(1, 4)]:
        xml.append(
            """            <output>
                <group>%s</group>
                <key>%s</key>
                <status>0x9F</status>
                <midino>%s</midino>
                <minimum>0.5</minimum>
            </output>"""
            % (
                get_group_name(channel + 1, light_mapping[knob]),
                get_key_name(light_mapping[knob]),
                midi[knob][j]["red"],
            )
        )

xml.append("<!-- Button lights -->")
# buttons (latched)
for j, channel in enumerate(DECK_ORDER):
    for latch in ["red", "orange", "green"]:
        for button in ["button%i" % i for i in range(1, 5)]:
            xml.append(
                """            <output>
                <group>%s</group>
                <key>%s</key>
                <status>0x9F</status>
                <midino>%s</midino>
                <minimum>0.5</minimum>
            </output>"""
                % (
                    get_group_name(channel + 1, light_mapping[button][latch]),
                    get_key_name(light_mapping[button][latch]),
                    midi[button][j][latch],
                )
            )

xml.append(
    """        </outputs>
    </controller>
</MixxxControllerPreset>"""
)

fh.writelines("\n".join(xml))
fh.close()
